import * as React from "react";
import { tween } from "./tween";
class SlideBox extends React.PureComponent {
    constructor(props) {
        super(props);
        this.slideDom = React.createRef();
        this.timer = 0;
        this.arr = [];
        this.state = {};
        this.queue = () => {
            this.arr.push(this.props.slide);
            if (!this.timer) {
                const slide = this.arr.shift();
                this.queueFn(slide);
            }
        };
        this.immediaFn = (slideDown) => {
            const { directionUp, slideFnCallback, duration, noPushEffect } = this.props;
            const dom = this.slideDom.current;
            const hasRun = !!this.timer;
            if (hasRun) {
                window.cancelAnimationFrame(this.timer);
                this.timer = 0;
            }
            dom.style.overflow = "hidden";
            if (slideDown && !hasRun) {
                dom.style.display = "block";
                dom.style.height = "0px";
            }
            const child = dom.firstElementChild;
            const origin = child.clientHeight;
            let start;
            let timeFrom = 0;
            const timeEnd = Math.ceil(duration / 17);
            if (hasRun) {
                start = parseInt(dom.style.height, 0);
            }
            else {
                start = slideDown ? 0 : origin;
            }
            const end = !slideDown ? 0 : origin;
            const dispaly = slideDown ? "block" : "none";
            const direct = directionUp ? "top" : "bottom";
            const originW = child.style.width;
            if (!noPushEffect) {
                child.style.position = "absolute";
                child.style[direct] = "0";
                child.style.width = "100%";
            }
            const fn = () => {
                this.timer = window.requestAnimationFrame(() => {
                    const h = tween.linear(timeFrom, start, end, timeEnd);
                    timeFrom++;
                    dom.style.height = `${h}px`;
                    if (timeFrom <= timeEnd) {
                        fn();
                    }
                    else {
                        dom.style.overflow = "initial";
                        dom.style.height = "";
                        dom.style.display = dispaly;
                        if (!noPushEffect) {
                            child.style.position = "";
                            child.style[direct] = "";
                            child.style.width = originW;
                        }
                        this.timer = 0;
                        slideFnCallback();
                    }
                });
            };
            if (!slideFnCallback(true)) {
                fn();
            }
        };
        this.slideFn = props.isImmedia ? this.immediaFn : this.queue;
    }
    componentDidMount() {
        if (!this.props.slide) {
            this.slideDom.current.style.display = "none";
        }
    }
    componentDidUpdate(preProps) {
        if (preProps.slide !== this.props.slide) {
            if (preProps.init !== this.props.init) {
                this.slideDom.current.style.display = this.props.slide ? "block" : "none";
            }
            else {
                this.slideFn(this.props.slide);
            }
        }
    }
    componentWillUnmount() {
        window.cancelAnimationFrame(this.timer);
    }
    queueFn(slideDown) {
        const { directionUp, duration, slideFnCallback, noPushEffect } = this.props;
        const dom = this.slideDom.current;
        dom.style.overflow = "hidden";
        if (slideDown) {
            dom.style.display = "block";
            dom.style.height = "0px";
        }
        const child = dom.firstElementChild;
        const origin = child.clientHeight;
        let timeFrom = 0;
        const timeEnd = Math.ceil(duration / 12);
        const dispaly = slideDown ? "block" : "none";
        const start = slideDown ? 0 : origin;
        const end = !slideDown ? 0 : origin;
        let direct, originW;
        if (!noPushEffect) {
            direct = directionUp ? "top" : "bottom";
            child.style.position = "absolute";
            originW = child.style.width;
            child.style.width = "100%";
            child.style[direct] = "0";
        }
        const fn = () => {
            this.timer = window.requestAnimationFrame(() => {
                const h = tween.easeInOutSine(timeFrom, start, end, timeEnd);
                timeFrom++;
                dom.style.height = `${h}px`;
                if (timeFrom <= timeEnd) {
                    fn();
                }
                else {
                    if (this.arr.length) {
                        const newSlide = this.arr.shift();
                        this.queueFn(newSlide);
                    }
                    else {
                        dom.style.display = dispaly;
                        dom.style.height = "";
                        if (!noPushEffect) {
                            child.style.position = "";
                            child.style[direct] = "";
                            child.style.width = originW;
                        }
                        this.timer = 0;
                        dom.style.overflow = "initial";
                        slideFnCallback();
                    }
                }
            });
        };
        if (!slideFnCallback(true)) {
            fn();
        }
    }
    render() {
        const { children, className, styleObj } = this.props;
        return (React.createElement("div", { ref: this.slideDom, className: `g-slideBox ${className || ""}`, style: styleObj }, children));
    }
}
SlideBox.defaultProps = {
    duration: 300,
    slideFnCallback: () => false
};
export default SlideBox;
