import * as Immutable from "immutable";
var calendarType;
(function (calendarType) {
    calendarType[calendarType["year"] = 1] = "year";
    calendarType[calendarType["searson"] = 2] = "searson";
    calendarType[calendarType["month"] = 3] = "month";
    calendarType[calendarType["day"] = 4] = "day";
})(calendarType || (calendarType = {}));
const timeStrValToTimeObjArr = function (isInit, props, curTime) {
    if (isInit) {
        const { month, searson, hour, minute } = curTime;
        return Array.from({ length: props.style }).map(() => ({
            year: "",
            month,
            searson,
            day: 1,
            hour,
            minute,
        }));
    }
    const { style, rotate, time, defaultTime } = props;
    let selTimeValStr = `${defaultTime}`.replace(/-/g, "");
    if (selTimeValStr) {
        const reg = /^\d{4}[0-9S: ]*$/;
        const valStrArr = selTimeValStr.split(",");
        if (style === 2 && valStrArr.length === 1) {
            valStrArr[1] = valStrArr[0];
            selTimeValStr = valStrArr.join(",");
        }
        const errorFormate = valStrArr.some((val) => !reg.test(val));
        if (errorFormate) {
            selTimeValStr = "";
        }
    }
    const defaultTimeArr = selTimeValStr.split(",");
    const curTimeArr = Array.from({ length: style }, () => Object.assign({}, curTime));
    const hour = curTime.hour;
    const minute = curTime.minute;
    const setTime = (item, curTimeItem) => {
        const year = Number(item.substr(0, 4));
        switch (rotate) {
            case calendarType.day: {
                const match = /^\d{4}[01]\d[0-3]\d/;
                if (!match.test(item)) {
                    return curTimeItem;
                }
                const month = Number(item.substr(4, 2));
                const timeStr = item.substr(8).trim();
                const timeArr = time && /[0-2]\d(:[0-5]\d){1,2}$/.test(timeStr) ? timeStr.split(":") : undefined;
                return {
                    year,
                    month,
                    searson: Math.ceil(month / 3),
                    day: Number(item.substr(6, 2)),
                    hour: timeArr ? Number(timeArr[0]) : hour,
                    minute: timeArr ? Number(timeArr[1]) : minute
                };
            }
            case calendarType.month: {
                const match = /^\d{4}[01]\d$/;
                if (!match.test(item)) {
                    return curTimeItem;
                }
                let month = Number(item.substr(4, 2));
                month = month < 1 ? 1 : month > 12 ? 12 : month;
                return {
                    year,
                    month,
                    searson: Math.ceil(month / 3),
                    day: 1,
                    hour,
                    minute
                };
            }
            case calendarType.searson: {
                const match = /^\d{4}S[1-4]$/;
                if (!match.test(item)) {
                    return curTimeItem;
                }
                const searson = Number(item.substr(5, 1));
                return {
                    year,
                    month: searson * 3 - 2,
                    searson,
                    day: 1,
                    hour,
                    minute
                };
            }
            case calendarType.year: {
                return {
                    year,
                    month: 1,
                    searson: 1,
                    day: 1,
                    hour,
                    minute
                };
            }
            default:
                return curTimeItem;
        }
    };
    const selTimeArr = curTimeArr.map((val, index) => !selTimeValStr ? val : setTime(defaultTimeArr[index], val));
    return selTimeArr;
};
const getInpTimeStrArr = function (selTimeArr, rotate, time) {
    const getStr = (val) => {
        const year = val.get("year");
        if (!year) {
            return "";
        }
        const month = `${val.get("month")}`.padStart(2, "0");
        switch (rotate) {
            case calendarType.day: {
                let timeStr = "";
                const day = `${val.get("day")}`.padStart(2, "0");
                if (time) {
                    const hour = `${val.get("hour")}`.padStart(2, "0");
                    const minute = `${val.get("minute")}`.padStart(2, "0");
                    timeStr = ` ${hour}:${minute}:00`;
                }
                return `${year}-${month}-${day}${timeStr}`;
            }
            case calendarType.searson: {
                const searson = `${val.get("searson")}`;
                return `${year}-S${searson}`;
            }
            case calendarType.year:
                return `${year}`;
            case calendarType.month:
                return `${year}-${month}`;
        }
    };
    const listArr = selTimeArr.map((val) => getStr(val)).toJS();
    return listArr[0] ? listArr : [""];
};
const getCurTime = function () {
    const time = new Date();
    const year = time.getFullYear();
    const month = time.getMonth() + 1;
    const day = time.getDate();
    const searson = Math.ceil(month / 3);
    const hour = time.getHours();
    const minute = time.getMinutes();
    return { year, searson, month, day, hour, minute };
};
const getLastYear = function (year) {
    let viewIndex = year % 10;
    viewIndex = viewIndex === 0 ? 10 : viewIndex;
    const startTime = year - viewIndex + 1;
    return startTime + 9;
};
const timeStrToNumber = function (strArr, valFormatt) {
    return valFormatt === "number" ? strArr.map((val) => val.replace(/-/g, "").replace(/:/g, "")
        .replace(/\s/g, "")) : strArr;
};
const compareTimeRang = (selTimeArr, viewIndex, showTimeObj, propsObj) => {
    const selTimeArrDate = selTimeArr.toJS();
    const { rotate, time, clickBefore, field, valFormatt } = propsObj;
    let status = false;
    selTimeArrDate[viewIndex] = showTimeObj;
    if (selTimeArrDate.length === 1 || !selTimeArr.getIn([0, "year"])) {
        status = false;
    }
    else {
        let startgtEnd = status;
        const startNode = selTimeArrDate[0];
        const endNode = selTimeArrDate[1];
        switch (rotate) {
            case calendarType.day:
                {
                    const mon1 = `${startNode.month}`.padStart(2, '0');
                    const day1 = `${startNode.day}`.padStart(2, '0');
                    const mon2 = `${endNode.month}`.padStart(2, '0');
                    const day2 = `${endNode.day}`.padStart(2, '0');
                    const startTime = `${startNode.year}${mon1}${day1}`;
                    const endTime = `${endNode.year}${mon2}${day2}`;
                    if (time) {
                        const hour1 = `${startNode.hour}`.padStart(2, '0');
                        const minute1 = `${startNode.minute}`.padStart(2, '0');
                        const hour2 = `${endNode.hour}`.padStart(2, '0');
                        const minute2 = `${endNode.minute}`.padStart(2, '0');
                        startgtEnd = startTime + `${hour1}${minute1}` > endTime + `${hour2}${minute2}`;
                    }
                    else {
                        startgtEnd = startTime > endTime;
                    }
                }
                break;
            case calendarType.month:
                {
                    const mon1 = `${startNode.month}`.padStart(2, '0');
                    const mon2 = `${endNode.month}`.padStart(2, '0');
                    startgtEnd = `${startNode.year}${mon1}` > `${endNode.year}${mon2}`;
                }
                break;
            case calendarType.searson:
                {
                    const season1 = startNode.searson;
                    const season2 = endNode.searson;
                    startgtEnd = `${startNode.year}${season1}` > `${endNode.year}${season2}`;
                }
                break;
            case calendarType.year:
                {
                    startgtEnd = `${startNode.year}` > `${endNode.year}`;
                }
                break;
            default:
                break;
        }
        status = startgtEnd;
    }
    if (status) {
        alert('开始时间要小于结束时间！');
    }
    if (!status && clickBefore) {
        const newSelTimeArr = Immutable.fromJS(selTimeArrDate);
        const timeVal = getInpTimeStrArr(newSelTimeArr, rotate, time);
        const str = timeStrToNumber(timeVal, valFormatt).join(",");
        status = !!clickBefore(str, {
            field,
            rotate: rotate,
            valFormatt: valFormatt
        }, newSelTimeArr);
    }
    return status;
};
export { timeStrValToTimeObjArr, calendarType, getInpTimeStrArr, getCurTime, getLastYear, timeStrToNumber, compareTimeRang, };
