import * as React from "react";
import { Animate } from "../animate/index";
import * as Immutable from "immutable";
import ParMenu from "./ParMenu";
import { createImmutableMap } from "../util/createImmutaleMap";
import { ScrollBox } from "../scroll/index";
const formatter = function (data, obj) {
    const { defaultMenuId, idField, onlyOpenSelected } = obj;
    let selectedIndex = "";
    const newData = data.map((par, index) => {
        par.selected = false;
        par.children.forEach((sub, subIndex) => {
            const status = defaultMenuId === `${sub[idField]}`;
            sub.selected = status;
            if (status) {
                par.selected = true;
                selectedIndex = `${index},${subIndex}`;
            }
        });
        par.drop = onlyOpenSelected ? !defaultMenuId ? index === 0 : par.selected : true;
        return par;
    });
    return {
        data: newData,
        selected: selectedIndex
    };
};
class NavMenu extends React.PureComponent {
    constructor(prop) {
        super(prop);
        this.scrollBoxMethod = undefined;
        this.bindScrollboxFn = (method) => {
            this.scrollBoxMethod = method;
        };
        this.toggleSlide = (index) => {
            this.setState((pre) => {
                const drop = !pre.immutableData.getIn([index, "drop"]);
                return {
                    immutableData: pre.immutableData.setIn([index, "drop"], drop)
                };
            });
        };
        this.changeImmutableData = (callback) => {
            let selectStringIndex = "";
            this.setState((pre) => {
                const { immutableData, selected } = pre;
                const { data, selected: _selected } = callback(immutableData, selected);
                selectStringIndex = _selected;
                return {
                    immutableData: data,
                    selected: _selected,
                };
            }, () => {
                const { clickBack, data, } = this.props;
                if (clickBack) {
                    const indexArr = selectStringIndex.split(",");
                    const node = data[+indexArr[0]].children[+indexArr[1]];
                    clickBack(node);
                }
            });
        };
        this.state = this.initState(this.props);
    }
    static getDerivedStateFromProps(nextProps, preState) {
        if (nextProps.data !== preState.preData) {
            const { data, selected } = formatter(nextProps.data, {
                defaultMenuId: nextProps.defaultMenuId,
                idField: nextProps.idField,
                onlyOpenSelected: nextProps.onlyOpenSelected
            });
            return {
                preData: nextProps.data,
                immutableData: Immutable.fromJS(data),
                selected: selected,
            };
        }
        else if (nextProps.init !== preState.preInit) {
            const { data, selected } = formatter(nextProps.data, {
                defaultMenuId: nextProps.init.id,
                idField: nextProps.idField,
                onlyOpenSelected: nextProps.onlyOpenSelected
            });
            return {
                immutableData: Immutable.fromJS(data),
                preInit: nextProps.init,
                selected
            };
        }
        else {
            return null;
        }
    }
    initState(prop) {
        const { defaultMenuId, data, idField, textField, urlField, iconField, init, onlyOpenSelected } = prop;
        const obj = formatter(data, { idField: idField, defaultMenuId: defaultMenuId, onlyOpenSelected });
        return {
            selected: obj.selected,
            immutableData: Immutable.fromJS(obj.data),
            fieldObj: createImmutableMap({
                id: idField,
                text: textField,
                url: urlField,
                icon: iconField,
            }),
            preData: data,
            preInit: init,
        };
    }
    componentDidUpdate(preProps) {
        if (preProps.data !== this.props.data && this.scrollBoxMethod && this.props.expand) {
            this.scrollBoxMethod.initScroll();
        }
    }
    getParMenu() {
        const { immutableData, fieldObj } = this.state;
        const { idField, expand, link } = this.props;
        return immutableData.map((node, index) => (React.createElement(ParMenu, { key: node.get(idField), node: node, link: link, scrollInitFn: this.scrollBoxMethod, expand: expand, fieldObj: fieldObj, parIndex: index, toggleSlide: this.toggleSlide, changeData: this.changeImmutableData })));
    }
    render() {
        const parItem = this.getParMenu();
        const { width, children, expand } = this.props;
        const menuCom = expand ? (React.createElement(ScrollBox, { bindIntiScroll: this.bindScrollboxFn },
            React.createElement("ul", { className: "g-menu" }, parItem))) : (React.createElement("ul", { className: "g-menu" }, parItem));
        return (React.createElement(Animate, { className: "g-slideMenu " + (!expand ? "expand" : ""), duration: 240, display: "flex", animation: { width: expand ? (width || 250) : 70 } },
            children,
            menuCom));
    }
}
NavMenu.defaultProps = {
    textField: "name",
    idField: "id",
    iconField: "icon",
    urlField: "url",
    width: 250
};
export default NavMenu;
