import * as React from "react";
import TrItem from "./TrItem";
import { SvgIcon } from "../../my-icon/index";
import { ScrollBox } from "../../scroll/index";
class TabView extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.state = {};
        this.colHeadRef = React.createRef();
        this.tabBodyRef = React.createRef();
        this.scrollBoxMethod = undefined;
        this.bindScrollboxFn = (method) => {
            this.scrollBoxMethod = method;
        };
        this.originScrollFn = (e) => {
            if (!e.currentTarget.classList.contains("action-body")) {
                return;
            }
            const { changeScrollTop, viewIndex } = this.props;
            const dom = e.currentTarget;
            const left = dom.scrollLeft;
            const colHead = this.colHeadRef.current;
            colHead.scrollLeft = left;
            changeScrollTop(dom.scrollTop, viewIndex);
        };
        this.wheelFn = (e) => {
            e.preventDefault();
            const { changeScrollTop, viewIndex } = this.props;
            const top = this.tabBodyRef.current.scrollTop;
            const step = e.deltaY > 0 ? 20 : -20;
            const distance = top + step;
            this.tabBodyRef.current.scrollTop = distance;
            const realDistance = this.tabBodyRef.current.scrollTop;
            changeScrollTop(realDistance, viewIndex);
        };
        this.makeSign = (e) => {
            e.preventDefault();
            const type = e.type;
            const dom = e.currentTarget;
            if (type === "mouseenter") {
                dom.classList.add("action-body");
            }
            else if (type === "mouseleave") {
                dom.classList.remove("action-body");
            }
        };
        this.checkAll = (e) => {
            const dom = e.currentTarget;
            const status = dom.dataset.status;
            this.props.changeState(status, "checkPar");
        };
        this.scrollCallback = (value, isVertical) => {
            if (isVertical) {
                const { changeScrollTop, viewIndex } = this.props;
                changeScrollTop(-value, viewIndex);
            }
            else {
                const colHead = this.colHeadRef.current;
                colHead.scrollLeft = -value;
            }
        };
    }
    overBox() {
        const tbodyDom = this.tabBodyRef.current;
        const { config: { forzen }, } = this.props;
        if (!forzen && tbodyDom.scrollHeight > tbodyDom.clientHeight) {
            const child = this.colHeadRef.current
                .firstElementChild;
            child.style.paddingRight = "18px";
        }
    }
    getCheckAll() {
        const { data } = this.props;
        const allChecked = data.every((val) => {
            const status = val.get("checked");
            return status;
        });
        const hasCheck = data.some((val) => val.get("checked"));
        const status = allChecked
            ? "checkbox-marked"
            : hasCheck
                ? "checkbox-has-selected"
                : "checkbox-blank";
        return (React.createElement("span", { onClick: this.checkAll, "data-status": allChecked },
            React.createElement(SvgIcon, { className: status })));
    }
    gethead() {
        const { theadChild, fixObj: { noOrder, multiply }, viewIndex } = this.props;
        return (React.createElement("div", { className: "tab-head" },
            React.createElement("table", null,
                React.createElement("thead", null,
                    React.createElement("tr", null,
                        multiply && viewIndex == 0 ? React.createElement("th", { className: "td-border" }, this.getCheckAll()) : null,
                        noOrder || viewIndex !== 0 ? null : React.createElement("th", { className: "td-border", style: { width: 60 } }, "\u5E8F\u53F7"),
                        theadChild)))));
    }
    createColgroup() {
        const { config: { child: cols }, } = this.props;
        const arr = cols.map((val) => {
            const { width, field } = val;
            const style = width ? { width } : undefined;
            return React.createElement("col", { style: style, key: field });
        });
        return React.createElement("colgroup", null, arr);
    }
    componentDidMount() {
        const { config, setTabBodyDom, viewIndex } = this.props;
        const dom = this.tabBodyRef.current;
        if (!dom) {
            return;
        }
        if (!config.forzen) {
            this.overBox();
        }
        setTabBodyDom(dom, viewIndex, this.scrollBoxMethod);
        const { current } = this.tabBodyRef;
        if (config.forzen) {
            current.addEventListener("wheel", this.wheelFn);
        }
    }
    componentWillUnmount() {
        const { config } = this.props;
        if (config.forzen) {
            this.tabBodyRef.current.removeEventListener("wheel", this.wheelFn);
        }
    }
    getBody() {
        const { data, fixObj, fixObj: { idField }, config, viewIndex, changeState, startIndex, originScroll } = this.props;
        const { child: cols } = config;
        const trs = data.map((val, index) => {
            const id = val.get(idField);
            const isMainView = viewIndex === 0;
            return (React.createElement(TrItem, { key: id, node: val, index: `${index + 1 + startIndex}`, changeState: changeState, cols: cols, isMainView: isMainView, fixObj: fixObj }));
        });
        const colgroup = this.createColgroup();
        const fn = !config.forzen && originScroll ? this.originScrollFn : undefined;
        const makeSignFn = !config.forzen && originScroll ? this.makeSign : undefined;
        return (React.createElement("div", { className: "tab-body-main", onScroll: fn, onMouseEnter: makeSignFn, onMouseLeave: makeSignFn, ref: this.tabBodyRef },
            React.createElement("table", null,
                colgroup,
                React.createElement("tbody", null, trs))));
    }
    componentDidUpdate(preProps) {
        if (preProps.isreLoad !== this.props.isreLoad && this.scrollBoxMethod) {
            this.scrollBoxMethod.initScroll();
        }
    }
    render() {
        const { config: { width, forzen }, originScroll } = this.props;
        const styleObj = width ? forzen ? { width } : { width, flex: "none" } : undefined;
        const className = forzen ? "tab-view-fix" : "tab-view-scroll";
        const body = forzen || originScroll ? React.createElement("div", { className: "tab-body-wrap" }, this.getBody()) : (React.createElement(ScrollBox, { scrollWidthAuto: true, scrollCallback: this.scrollCallback, bindIntiScroll: this.bindScrollboxFn }, this.getBody()));
        return (React.createElement("div", { className: `${className} tab-viewBody`, style: styleObj },
            React.createElement("div", { className: "tab-head-wrap", ref: this.colHeadRef }, this.gethead()),
            body));
    }
}
export default TabView;
