import * as React from "react";
import { SvgIcon } from "../../my-icon/index";
import { activeStatus } from "./formatterTreeData";
class Item extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.state = {};
        this.slideIconEvent = (e) => {
            e.stopPropagation();
            const index = e.currentTarget.parentElement.parentElement.dataset.index;
            this.props.changeState(index, "expand");
        };
        this.clickFn = (e) => {
            const index = e.currentTarget.dataset.index;
            const { changeState } = this.props;
            changeState(index, "expand");
        };
        this.checkFn = (e) => {
            e.stopPropagation();
            const dom = e.currentTarget;
            const index = dom.parentElement.dataset.index;
            const { changeState, isPar } = this.props;
            changeState(index, isPar ? "checkPar" : "active");
        };
        this.changeSelect = (e) => {
            const index = e.currentTarget.dataset.index;
            this.props.changeState(index, "active");
        };
    }
    getCheck() {
        const { fixObj: { multiply }, } = this.props;
        if (multiply) {
            const { node } = this.props;
            const active = node.get("active") === activeStatus.hasSelect
                ? "checkbox-has-selected"
                : node.get("active") === activeStatus.noSelect
                    ? "checkbox-blank"
                    : "checkbox-marked";
            return React.createElement(SvgIcon, { className: active });
        }
        else {
            return undefined;
        }
    }
    getIcon() {
        const { node, isPar, fixObj: { itemIcon = "file", multiply }, } = this.props;
        if (isPar) {
            const expand = node.get("expand");
            return (React.createElement(React.Fragment, null,
                React.createElement("span", { onClick: multiply ? this.slideIconEvent : undefined },
                    React.createElement(SvgIcon, { className: expand ? "minus" : "plus" })),
                React.createElement(SvgIcon, { className: expand ? "folder-open" : "folder" })));
        }
        else {
            return typeof itemIcon === "boolean" ? undefined : React.createElement(SvgIcon, { className: typeof itemIcon === "function" ? itemIcon(node) : itemIcon });
        }
    }
    getFirstText(text, showOrder) {
        const { lev, isPar, index, order, fixObj: { multiply }, } = this.props;
        const fn = isPar ? this.clickFn : undefined;
        const className = isPar ? "tree-par" : undefined;
        const check = multiply ? this.checkFn : undefined;
        const checkName = multiply ? "tree-check txt-content" : "txt-content";
        return (React.createElement("div", { onClick: fn, className: className, "data-index": index },
            React.createElement("span", { onClick: check, className: checkName },
                React.createElement("span", { style: { paddingLeft: lev * 14 } },
                    this.getCheck(),
                    this.getIcon(),
                    showOrder ? React.createElement("span", { className: `file-order ${isPar ? "par" : ""}` }, order) : null,
                    text))));
    }
    render() {
        const { cols, node, fixObj: { tabField, noOrder, multiply, slectField, fileOrder }, isMainView, order, isPar, index } = this.props;
        const tds = cols.map((td, oindex) => {
            const { field, formatter, align } = td;
            const text = formatter
                ? formatter(node, oindex, tabField, field, order)
                : node.get(field);
            let str;
            let alignName = align ? `${align}` : "";
            if (isMainView) {
                const showFileOrder = !!(!formatter && fileOrder);
                if (noOrder) {
                    str = oindex === 0 ? this.getFirstText(text, showFileOrder) : text;
                }
                else {
                    str =
                        oindex === 0
                            ? node.get("order")
                            : oindex === 1
                                ? this.getFirstText(text, showFileOrder)
                                : text;
                    alignName = oindex === 1 ? "" : alignName;
                }
            }
            else {
                str = text;
            }
            return (React.createElement("td", { key: field, className: `td-border ${alignName}`, "data-index": !multiply && !isPar ? index : undefined, "data-field": field, onClick: !multiply && !isPar && (!slectField ? true : slectField === field) ? this.changeSelect : undefined }, str));
        });
        const selected = !isPar && node.get("active") == activeStatus.select ? activeStatus.select : "";
        return React.createElement("tr", { className: `render-td ${selected}` }, tds);
    }
}
export default Item;
