import * as Immutable from "immutable";
var activeStatus;
(function (activeStatus) {
    activeStatus["select"] = "selected";
    activeStatus["hasSelect"] = "hasSelect";
    activeStatus["noSelect"] = "noSelect";
})(activeStatus || (activeStatus = {}));
const formatterTreeData = function (fixObj, defaultVal, data) {
    const { idField: id, childField, multiply } = fixObj;
    let defaultValArr = defaultVal.split(",");
    if (!multiply && defaultValArr.length > 1) {
        defaultValArr.length = 1;
    }
    let oldSelectedIndex = "";
    let listSelected = Immutable.List([]);
    let oindex = 0;
    let prePath = [];
    const immutableData = Immutable.fromJS(data, function (_key, val, path) {
        if (Immutable.isKeyed(val)) {
            let node = val.toOrderedMap();
            if (path.some((str) => str !== childField && typeof str !== "number")) {
                return node;
            }
            let children = node.get(childField);
            if (!children) {
                children = Immutable.List([]);
                node = node.set(childField, children);
            }
            let active;
            if (children.size) {
                if (multiply) {
                    const hasSelected = children.some((vals) => vals.get("active") === activeStatus.hasSelect);
                    if (hasSelected) {
                        active = activeStatus.hasSelect;
                    }
                    else {
                        let selectCount = children.reduce((cur, vals) => {
                            let total = cur;
                            if (vals.get("active") === activeStatus.select) {
                                total++;
                            }
                            return total;
                        }, 0);
                        const childSize = children.size;
                        if (selectCount === childSize) {
                            active = activeStatus.select;
                        }
                        else if (selectCount === 0) {
                            active = activeStatus.noSelect;
                        }
                        else {
                            active = activeStatus.hasSelect;
                        }
                    }
                }
                else {
                    active = activeStatus.noSelect;
                }
            }
            else {
                const isDefault = defaultValArr.includes(`${node.get(id)}`);
                active = isDefault ? activeStatus.select : activeStatus.noSelect;
                if (isDefault) {
                    const idVal = node.get(id);
                    listSelected.push(idVal);
                    if (!multiply) {
                        oldSelectedIndex = path.join(",");
                    }
                }
            }
            oindex += 1;
            let countIndex;
            const parArr = prePath.slice(0, prePath.length - 2);
            if (path.join(",") === parArr.join(",")) {
                countIndex = children.get(0).get("order") - 1;
            }
            else {
                countIndex = oindex + (path.length - 1) / 2;
            }
            prePath = path;
            node = node.set("order", countIndex);
            const { noexpand } = fixObj;
            const expand = typeof noexpand == "string"
                ? noexpand.split(",").includes(String(node.get("id")))
                : !noexpand;
            node = node.withMutations((map) => map.set("active", active).set("expand", expand));
            return node;
        }
        else {
            return val.toList();
        }
    });
    return {
        data: immutableData,
        oldSelectedIndex: oldSelectedIndex,
        selectArr: listSelected,
    };
};
const cascade = function (paths, data, childField) {
    let _data = data;
    const pathArr = paths.split(",");
    pathArr.pop();
    _data = _data.withMutations((list) => {
        let _list = list;
        while (pathArr.length) {
            const path = pathArr.join(`,${childField},`).split(",");
            _list = _list.updateIn(path, (nodes) => {
                let _node = nodes;
                const child = _node.get(childField);
                const hasSelect = child.some((val) => val.get("active") === activeStatus.hasSelect);
                let active;
                if (hasSelect) {
                    active = activeStatus.hasSelect;
                }
                else {
                    const selectCount = child.reduce((cur, val) => {
                        let total = cur;
                        if (val.get("active") === activeStatus.select) {
                            total++;
                        }
                        return total;
                    }, 0);
                    const childSize = child.size;
                    if (selectCount === childSize) {
                        active = activeStatus.select;
                    }
                    else if (selectCount === 0) {
                        active = activeStatus.noSelect;
                    }
                    else {
                        active = activeStatus.hasSelect;
                    }
                }
                return _node.set("active", active);
            });
            pathArr.pop();
        }
        return _list;
    });
    return _data;
};
export { formatterTreeData, cascade, activeStatus };
